@@ -13,7 +13,7 @@ import logging
 import datetime as dt
 import traceback
 
-from google.protobuf import timestamp_pb2
+from google.protobuf import json_format, timestamp_pb2
 
 logging.basicConfig(
     level=logging.INFO,
@@ -51,6 +51,7 @@ from service_pb2_grpc import OrcaProcessorServicer
 from orca_python import envs
 from orca_python.exceptions import (
     InvalidDependency,
+    InvalidWindowArgument,
     InvalidAlgorithmArgument,
     InvalidAlgorithmReturnType,
 )
@@ -136,11 +137,41 @@ class NoneResult:
 returnResult = StructResult | ArrayResult | ValueResult | NoneResult
 
 
+@dataclass
+class Window:
+    time_from: dt.datetime
+    time_to: dt.datetime
+    name: str
+    version: str
+    origin: str
+    metadata: Dict[str, Any] = field(default_factory=dict)
+
+
 @dataclass
 class ExecutionParams:
-    window: pb.Window
+    window: Window
     dependencies: Optional[Iterable[pb.AlgorithmResult]] = None
 
+    def __init__(
+        self,
+        window: Window | pb.Window,
+        dependencies: Optional[Iterable[pb.AlgorithmResult]] = None,
+    ):
+        if isinstance(window, Window):
+            self.window = window
+        elif isinstance(window, pb.Window):
+            self.window = Window(
+                time_from=window.time_from.ToDatetime(),
+                time_to=window.time_to.ToDatetime(),
+                name=window.window_type_name,
+                version=window.window_type_version,
+                origin=window.origin,
+                metadata=json_format.MessageToDict(window.metadata),
+            )
+        else:
+            raise InvalidWindowArgument(f"window of type {type(window)} not handled")
+        self.dependencies = dependencies
+
 
 class AlgorithmFn(Protocol):
     def __call__(
@@ -151,16 +182,6 @@ class AlgorithmFn(Protocol):
 T = TypeVar("T", bound=AlgorithmFn)
 
 
-@dataclass
-class Window:
-    time_from: dt.datetime
-    time_to: dt.datetime
-    name: str
-    version: str
-    origin: str
-    metadata: Dict[str, Any] = field(default_factory=dict)
-
-
 def EmitWindow(window: Window) -> None:
     """
     Emits a window to Orca-core.
@@ -177,8 +198,8 @@ def EmitWindow(window: Window) -> None:
     _time_to.FromDatetime(window.time_to)
 
     window_pb = pb.Window()
-    window_pb.time_to = _time_from
-    window_pb.time_from = _time_to
+    window_pb.time_to.CopyFrom(_time_to)
+    window_pb.time_from.CopyFrom(_time_from)
     window_pb.window_type_name = window.name
     window_pb.window_type_version = window.version
     window_pb.origin = window.origin
